/**
 * Classe BootLoader per a sincronitzar els arxius font
 *
 * @version 1.2.0
 * @param {bool} debug
 * @constructor
 */
function BootLoader (debug)
{
    var me = this;

    /**
     * Nom dels directoris WWW i BACKUP i nom del fitxer INDEX de la App
     * @type {string}
     */
    me.WWW_DIR = 'www';
    me.BACKUP_DIR = 'www.backup';
    me.INDEX_FILE = 'index.html';

    /**
     * Mode debug
     * @type {boolean}
     */
    me.debug = !!debug;

    /**
     * Idioma del dispositiu
     * @type {string}
     */
    me.language = '';

    /**
     * Informació sobre els SRC actuals
     * @type {object}
     */
    me.currentSRC = {
        exists: false,
        version: ''
    };

    /**
     * Instància del FileSystemManager i del Directori on desarem els SRC
     * @type {null}
     */
    me.fs = null;
    me.dataDirectory = null;

    /**
     * Variables vàries per la descàrrega
     * @type {object}
     */
    me.download = {

        /**
         * Instància FileTransfer/XHR
         * @type {FileTransfer|XMLHttpRequest}
         */
        fileTransfer: null,

        /**
         * Informació sobre l'estat de la descàrrega
         * @type {string}
         */
        state: '',

        /**
         * Cua de fitxers a descarregar
         * @type {Array}
         */
        fileQueue: [],

        /**
         * Quantitat de fitxers (progrés)
         * @type {int}
         */
        fileQueueLength: 0,

        /**
         * Mida total dels fitxers (utilitzat en el progrés)
         * @type {int}
         */
        totalSize: 0,

        /**
         * Mida descarregada (utilitzat en el progrés)
         */
        downloadedSize: 0
    };

    /**
     * Referències jQuery
     * @type {jQuery}
     */
    me.$bootloader = $('#bootloader');
    me.$progressWrap = me.$bootloader.find('.progress-bar-wrap');
    me.$progress = me.$bootloader.find('.progress-bar');
    me.$feedback = me.$bootloader.find('.feedback');
    me.$buttonCancel = me.$bootloader.find('.download-cancel');
}

/**
 * Log a BootLoader message to console if debug is activated
 *
 * @private
 * @param {string} message
 * @param {string} [type=log]
 */
BootLoader.prototype._log = function(message, type)
{
    if (this.debug && typeof console !== 'undefined')
    {
        switch (type)
        {
            case 'error':
                console.error('[BL_ERR]: ' + message);
                break;

            case 'log':
            default:
                console.log('[BL_LOG]: ' + message);
                break;
        }
    }
};

/**
 * Alertar l'usuari quan no hi ha conexió i la necessitem per força
 *
 * @private
 */
BootLoader.prototype._alertNoConnection = function()
{
    var me = this;

    if (IS_APP)
    {
        navigator.notification.confirm(
            me.__('Internet connection is needed in order to initialize the application.'),
            function(btnId)
            {
                // Sortir de la app si podem
                if (btnId === 1)
                {
                    me._log('Check Update: No internet connection, No SRC, Alert user, Exit');

                    if(navigator.app)
                    {
                        navigator.app.exitApp();
                    }
                }
                // Reintentar
                else
                {
                    me._log('Check Update: No internet connection, No SRC, Alert user, Retry');

                    me.checkUpdate();
                }
            },
            me.__('No connection'),
            [me.__('Exit'), me.__('Retry')]
        );
    }
    else
    {
        if (confirm(me.__('Internet connection is needed in order to initialize the application.')))
        {
            me._log('Check Update: No internet connection, No SRC, Alert user, Retry');
            me.checkUpdate();
        }
        else
        {
            me._log('Check Update: No internet connection, No SRC, Alert user, Exit');
        }
    }
};

/**
 * Inicialitza i fer quatre comprovacions
 */
BootLoader.prototype.__ = function(msg)
{
    var me = this;

    if (BootLoader.i18n && BootLoader.i18n[me.language] && BootLoader.i18n[me.language][msg])
    {
        return BootLoader.i18n[me.language][msg];
    }

    return msg;
};

/**
 * Inicialitza i fer quatre comprovacions
 */
BootLoader.prototype.initialize = function()
{
    var me = this;

    // Desar el "document_root" per a després utilitzar-lo per a carregar cordova.js i altres llibreries incloses a l'APK/IPA
    localStorage.setItem('DOCUMENT_ROOT', window.location.href.replace(/\/[^\/]*$/, ''));

    // Obtenir l'idioma preferit del dispositiu, si coincideix amb algun dels nostres, sino anglès
    getPreferredLanguages(function(languages){

        var i, k,
            lang = '';

        if (typeof BootLoader.i18n !== 'object')
        {
            BootLoader.i18n = {};
        }

        // Try to find best match from device default languages
        i = 0;
        while (!lang && i < languages.length)
        {
            for (k in BootLoader.i18n)
            {
                if (BootLoader.i18n.hasOwnProperty(k) && languages[i] == k)
                {
                    lang = k;
                    break;
                }
            }

            i++;
        }

        me.language = lang;

        // Event del botó d'abortar que es mostra quan ja tenim SRC
        me.$buttonCancel
            .html(me.__('Cancel'))
            .on('click', function(event){
                me.$buttonCancel.prop('disabled', true);
                me.downloadAbort();
            });

        me._log('Initialize ... Done');

        me.begin();
    });
};


/**
 * Comença la sincronització
 */
BootLoader.prototype.begin = function()
{
    var me = this;

    me._log('Begin syncronization');

    // Crear instància FileSystemManager
    me.fs = new FileSystemManager(me.debug);

    // Obtenir directory on desar els fitxers (directory DATA privat)
    me.fs.getDirectoryByCategory(
        FileSystemCategory.DATA,
        true,
        function(dataDirectory)
        {
            me.dataDirectory = dataDirectory;

            me.checkCurrentVersion(function(){
                me.checkUpdate();
            });
        },
        function(error)
        {
            // TODO: ERROR FS
        }
    );
};

/**
 * Agafar info sobre la versió actual instalada (si n'hi ha).
 *
 * @param {function} onComplete
 */
BootLoader.prototype.checkCurrentVersion = function (onComplete)
{
    var me = this;

    me._log('[SYNC] Check current version');

    me.fs.fileExists(
        me.dataDirectory,
        '/' + me.WWW_DIR + '/' + me.INDEX_FILE,
        function(exists)
        {
            me.currentSRC.version = localStorage.getItem('SOURCES_VERSION') || '';
            me.currentSRC.exists = exists && me.currentSRC.version != '';

            me._log('Check current version ... Done (exists: '+me.currentSRC.exists+', version: '+me.currentSRC.version+')');

            if (typeof onComplete === 'function')
            {
                onComplete();
            }
        }
    );

    return me;
};

/**
 * Comprovar si existeix alguna versió més nova al servidor
 */
BootLoader.prototype.checkUpdate = function()
{
    var me = this;

    // Comprovar que tenim conexió per a fer comprovació d'actualització
    if (checkDeviceConnection())
    {
        me._log('Check Update: Got internet connection');

        // Enviar la versió actual
        var postData = {
            version: {
                major: APP.VERSION.MAJOR,
                full: me.currentSRC.version
            },
            debug: me.debug ? 1 : 0
        };

        // Comprovar si hi ha actualització
        getJSON(
            SERVER.WS + '/loader',
            postData,
            true,
            function(json)
            {
                // No tinc cap versió instal·lada o versió servidor > versió nostra? Toca actualitzar
                if (!me.currentSRC.exists || versionCompare(me.currentSRC.version, json.version, '<'))
                {
                    me._log('Check Update: Update needed (current: "'+me.currentSRC.version+'", new: "'+json.version+'")');

                    // Amagar splash screen per a mostrar el progrés (i el botó si cal)
                    if (IS_APP && navigator.splashscreen)
                    {
                        navigator.splashscreen.hide();
                    }

                    // Descarregar nous sources i engegar APP en finalitzar
                    me.initDownload(json.sources);
                    me.startDownload(
                        function()
                        {
                            localStorage.setItem('SOURCES_VERSION', json.version);

                            me.currentSRC.version = json.version;
                            me.currentSRC.exists = true;

                            me.launchApp();
                        }
                    );
                }
                // Ja estem a la última versió, engegar
                else
                {
                    me._log('Check Update: No update needed (current: "'+me.currentSRC.version+'", server: "'+json.version+'")');
                    me.launchApp();
                }

            },
            function(textStatus, errorThrown)
            {
                // Si no hi ha conexió però tenim alguna versió instalada, engegar-la
                if (me.currentSRC.exists)
                {
                    me._log('Check Update: Conection error, Got SRC, Launching App');

                    me.launchApp();
                }
                // Si no hi ha conexió, i no tenim cap versió instal·lada, alertar
                else
                {
                    me._log('[SYNC] Check Update: Conection error, No SRC, Alert user');

                    // Mostrar diàleg usuari
                    me._alertNoConnection();
                }
            }
        );
    }
    // Si no hi ha conexió però tenim alguna versió instalada, engegar-la, ja actualitzarem en un altre moment
    else if (me.currentSRC.exists)
    {
        me._log('Check Update: No internet connection, Got SRC, Launching App');

        me.launchApp();
    }
    // Si no hi ha conexió, i no tenim cap versió instal·lada, alertar
    else
    {
        me._log('Check Update: No internet connection, No SRC, Alert user');

        // Mostrar diàleg usuari
        me._alertNoConnection();

    }
};


/**
 * Inicialitza descàrrega de SRC
 *
 * @param {object} sources Objecte amb informació dels SRC a baixar-se
 */
BootLoader.prototype.initDownload = function (sources)
{
    var me = this;

    me._log('Download initialization');

    // Inicialitzar variables
    me.download.fileTransfer = null;
    me.download.fileQueue = [];
    me.download.fileQueueLength = 0;
    me.download.totalSize = 0;
    me.download.downloadedSize = 0;

    // Mostrar barra de progrés
    me.$progress.css('width', '0%');
    me.$progressWrap.show();

    // Mostrar botó cancelar (només si ja tenim algun SRC)
    if (me.currentSRC.exists)
    {
        me.$feedback.show();
    }

    // Buclejar arxius per generar cua i alguns valors
    for (var i = 0, l = sources.files.length; i < l; i++)
    {
        me.download.fileQueue.push({
            source: SERVER.BASE + sources.path + sources.files[i].path + '/' + sources.files[i].name,
            target: '/' + me.WWW_DIR + sources.files[i].path + '/' + sources.files[i].name,
            name: sources.files[i].name,
            size: sources.files[i].size
        });

        me.download.totalSize += sources.files[i].size;
    }

    // Desar mida de la cua
    me.download.fileQueueLength = me.download.fileQueue.length;

    // Estat ready
    me.download.state = 'ready';
};

/**
 * Copia la versió actual dels SRC a un directori backup i crea el directori dels SRC.
 *
 * @param {function} onComplete
 */
BootLoader.prototype.prepareDownload = function (onComplete)
{
    var me = this;

    me._log('Download preparation');

    // Esborrar backup vell si n'hi ha
    me.fs.rmDir(
        me.dataDirectory,
        me.BACKUP_DIR,
        true,
        function()
        {
            // Obrir directori dels SRC
            me.fs.openDir(
                me.dataDirectory,
                me.WWW_DIR,
                'a',
                function(sourcesDirEntry)
                {
                    // Moure els SRC al directori Backup
                    sourcesDirEntry.moveTo(
                        me.dataDirectory,
                        me.BACKUP_DIR, function()
                        {
                            // Crear directori dels SRC
                            me.fs.openDir(
                                me.dataDirectory,
                                me.WWW_DIR,
                                'a',
                                function()
                                {
                                    me._log('Download preparation... Done');

                                    if (typeof onComplete === 'function')
                                    {
                                        onComplete();
                                    }
                                }
                            );
                        },
                        function()
                        {
                            // Mai hauria d'entrar aquí
                        }
                    );
                }
            );
        }
    );
};

/**
 * Iniciar la descàrrega fitxers
 *
 * @param {function} onComplete
 */
BootLoader.prototype.startDownload = function (onComplete)
{
    var me = this;

    me._log('Download start');

    // Preparar directoris
    me.prepareDownload(
        function()
        {
            // Estat
            me.download.state = 'downloading';

            // Descarregar els documents un darrera l'altre
            me.downloadNext(function(){

                me.downloadClean(function(){

                    me._log('Download completed');

                    me.$feedback.hide();

                    if (typeof onComplete === 'function')
                    {
                        onComplete();
                    }
                });
            });
        }
    );
};

/**
 * Processa la cua de fitxers a descarregar
 *
 * @param {function} onComplete
 */
BootLoader.prototype.downloadNext = function (onComplete)
{
    var me = this;

    // Si encara queden docs, descarregar següent
    if (me.download.fileQueue.length)
    {
        var file = me.download.fileQueue.shift();

        me._log('Download next file: '+file.name);

        // Descarregar fitxer i desar instància de baixada per a poder abortar-la
        me.download.fileTransfer = new FileTransferManager(me.fs, me.debug);
        me.download.fileTransfer.download(
            file.source,
            me.dataDirectory,
            file.target,
            function(event)
            {
                if (event.lengthComputable)
                {
                    me.downloadProgress(me.download.fileQueueLength, me.download.fileQueueLength - me.download.fileQueue.length, me.download.totalSize, me.download.downloadedSize, file.size, file.size);
                }
            },
            function(fileEntry)
            {
                me._log('Download next file: '+file.name+' ... Done');

                // Ja no podem abortar, posar a null per evitar errors
                me.download.fileTransfer = null;

                // Sumar a la mida ja descarregada la del fitxer actual
                me.download.downloadedSize += file.size;

                // Cridar la funció progrés
                me.downloadProgress(me.download.fileQueueLength, me.download.fileQueueLength - me.download.fileQueue.length, me.download.totalSize, me.download.downloadedSize, file.size, file.size);

                // Descarregar següent
                if (me.download.state === 'downloading')
                {
                    me.downloadNext(onComplete);
                }
            },
            function()
            {
                // Ja no podem abortar, posar a null per evitar errors
                me.download.fileTransfer = null;

                // Si no hem abortat (abortar fa que cridi el callback error amb l'error FileTransferError.ABORT_ERR)
                if (me.download.state !== 'aborted')
                {
                    me._log('Download next file: '+file.name+' ... Error');

                    var dlgBtns = [];

                    dlgBtns.push(me.__('Exit'));

                    if (me.currentSRC.exists)
                    {
                        dlgBtns.push(me.__('Start up'));
                    }

                    dlgBtns.push(me.__('Retry'));

                    // Mostrar dialeg amb error i opcions
                    navigator.notification.confirm(
                        me.__('There was an error while downloading application files.'),
                        function(btnId)
                        {
                            // Rollback + exit
                            if (btnId == 1)
                            {
                                me.downloadRollback(function(){
                                    if(navigator.app)
                                    {
                                        navigator.app.exitApp();
                                    }
                                });
                            }
                            // Rollback + engegar app
                            else if (me.currentSRC.exists && btnId == 2)
                            {
                                me.downloadRollback(function(){
                                    me.launchApp();
                                });
                            }
                            // Reintentar
                            else
                            {
                                me.downloadNext(onComplete);
                            }
                        },
                        me.__('Download error'),
                        dlgBtns
                    );
                }
            }
        );
    }
    // Si no queden més docs, cridar onComplete();
    else
    {
        me.download.state = 'completed';

        if (typeof onComplete === 'function')
        {
            // Una mica de retràs perquè es vegi el 100%
            setTimeout(onComplete, 500);
        }
    }
};

/**
 * Callback per al progrés de descàrrega
 *
 * @param {int} totalFiles
 * @param {int} downloadedFiles
 * @param {int} totalSize
 * @param {int} totalDownloadedSize
 * @param {int} fileSize
 * @param {int} fileDownloadedSize
 */
BootLoader.prototype.downloadProgress = function(totalFiles, downloadedFiles, totalSize, totalDownloadedSize, fileSize, fileDownloadedSize)
{
    var me = this;

    // Calcular % total
    var totalPercentage = (100 * totalDownloadedSize / totalSize).toFixed(1);

    // Actualitzar barra progrés
    me.$progress.css('width', totalPercentage + '%');
};

/**
 * Recupera el backup anterior
 *
 * @param {function} onComplete
 */
BootLoader.prototype.downloadRollback = function (onComplete)
{
    var me = this;

    me._log('Download rollback');

    // Esborrar directori dels fitxers baixats
    me.fs.rmDir(
        me.dataDirectory,
        me.WWW_DIR,
        true,
        function()
        {
            // Obrir directory del backup
            me.fs.openDir(
                me.dataDirectory,
                me.BACKUP_DIR,
                'a',
                function(sourcesBackupDirEntry)
                {
                    // Moure SRC del Backup al sources
                    sourcesBackupDirEntry.moveTo(
                        me.dataDirectory,
                        me.WWW_DIR,
                        function()
                        {
                            me._log('Download rollback ... Done');

                            if (typeof onComplete === 'function')
                            {
                                onComplete();
                            }
                        }
                    );
                }
            );
        }
    );
};

/*
 * Aborta la descàrrega
 */
BootLoader.prototype.downloadAbort = function()
{
    var me = this;

    // Només té sentit abortar mentre estem descarregant, ni abans ni després
    if (me.download.state === 'downloading')
    {
        me._log('Download abort');

        // Estat
        me.download.state = 'aborted';

        // Abortar descàrrega actual si n'hi ha
        if (me.download.fileTransfer)
        {
            me.download.fileTransfer.abort();
            me.download.fileTransfer = null;
        }

        // Buidar cua fitxers
        me.download.fileQueue = [];

        // Fer rollback i engegar App
        me.downloadRollback(function(){
            me.launchApp();
        });
    }
};

/**
 * Fa neteja
 *
 * @param {function} [onComplete]
 */
BootLoader.prototype.downloadClean = function(onComplete)
{
    var me = this;

    me.fs.rmDir(
        me.dataDirectory,
        me.BACKUP_DIR,
        true,
        function()
        {
            if (typeof onComplete === 'function')
            {
                onComplete();
            }
        }
    );
};


/**
 * Redirecciona l'aplicació cap als SRC descarregats
 */
BootLoader.prototype.launchApp = function()
{
    var me = this;

    me._log('Launching app ('+(IS_APP ? 'IS_APP' : '')+')');

    var redirectURL = me.dataDirectory.toURL().replace(/\/+$/, '') + '/' + me.WWW_DIR + '/' + me.INDEX_FILE;

    if (IS_APP)
    {
        // Mostrar splash pq sino surt la pantalla en blanc uns segons i no queda bé
        if(navigator.splashscreen){
        	navigator.splashscreen.show();
        }       

        //Redirigir als SRC baixats
        window.location.replace(redirectURL);
    }
    else
    {
        // Fer-ho així ja que sino s'obre en forma de popup en comptes de una pestanya (pestanya requereix acció de l'usuari, ex: click)
        me.$feedback.show().html('<a class="download-cancel" onclick="window.open(\''+redirectURL+'\', \'_blank\');">Obrir App en una nova pestanya</a>');

        /**
         * Obrir els SRC en finestra nova si és navegador
         * (Deprecated: Si ho obrim en popup, no ho fa en pestanya nova pq no hi ha acció de l'usuari. Millor mostrar un botó i fer click, total és debugging)
         * @deprecated
         */
        /*var win = window.open(redirectURL, '_blank');
        if (win)
        {
            win.focus();
        }*/
    }
};
