﻿/**
 * Retorna la traducció d'un text segons quin sigui el lang
 *
 * @param {string} msg Cadena de text a traduïr
 */
function __(msg)
{
    if (typeof BootLoader !== 'undefined' && i18n[language] && i18n[language][msg])
    {
        return i18n[msg];
    }

    return msg;
}

/**
 * Try to find user's device language using some methods (cordova Api, userAgent, ...)
 *
 * @param {function} onComplete
 */
function getPreferredLanguages (onComplete)
{
    var method, methods, callback,
        LANG_REGEX = /^([a-z]{2})([\-_]([a-z]{2}))?$/i;

    var parseLocaleString = function(locale)
    {
        var matches, i;
        var parts = locale.split(/\s*,\s*/);
        var languages = [];
        var lang = '';

        for (i = 0; i < parts.length; i++)
        {
            if (matches = parts[i].match(LANG_REGEX))
            {
                lang = matches[1].toLowerCase();

                if (languages.indexOf(lang) === -1)
                {
                    languages.push(lang);
                }
            }
        }

        return languages;
    };

    if (arguments.length === 1)
    {
        methods = ['cordova-preferred', 'cordova-locale', 'userAgent'];
        callback = arguments[0] || function(){};
    }
    else
    {
        methods = arguments[0] || [];
        callback = arguments[1] || function(){};
    }

    method = methods.splice(0, 1).shift();

    switch (method)
    {
        case 'cordova-preferred':
        case 'cordova-locale':
            var functionName = method === 'cordova-preferred' ? 'getPreferredLanguage' : 'getLocaleName';

            if (navigator && navigator.globalization && navigator.globalization[functionName])
            {
                navigator.globalization[functionName](
                    function (locale)
                    {
                        if (locale.value)
                        {
                            var languages = parseLocaleString(locale.value);

                            if (languages.length)
                            {
                                callback(languages);
                            }
                            else
                            {
                                getPreferredLanguages(methods, callback);
                            }
                        }
                        else
                        {
                            getPreferredLanguages(methods, callback);
                        }
                    },
                    function ()
                    {
                        getPreferredLanguages(methods, callback);
                    }
                );
            }
            else
            {
                getPreferredLanguages(methods, callback);
            }
            break;

        case 'userAgent':
            var languages = parseLocaleString(navigator.language);

            if (languages.length)
            {
                callback(languages);
            }
            else
            {
                getPreferredLanguages(methods, callback);
            }
            break;

        default:
            callback([]);
            break;
    }
}


/**
 * Compara dues versions en forma de String ("1.10.1" > "1.9"), ("1.2.0-dev" < "1.2.0")
 *
 * @param {string} v1 Versió 1
 * @param {string} v2 Versió 2
 * @param {string} [operator] Operator (Case sensitive) [<, lt, <=, le, >, gt, >=, ge, ==, =, eq, !=, <>, ne]
 * @return {int|bool} Returns (-1 = v1 < v2), (1 = v1 > v2), (0 = v1 === v2). When using the optional operator argument, the function will return TRUE if the relationship is the one specified by the operator, FALSE otherwise.
 */
function versionCompare(v1, v2, operator)
{
    var
        i = 0,
        x = 0,
        compare = 0,

        vm = {
            'dev': -6,
            'alpha': -5,
            'a': -5,
            'beta': -4,
            'b': -4,
            'RC': -3,
            'rc': -3,
            '#': -2,
            'p': 1,
            'pl': 1
        },

        prepVersion = function(v)
        {
            v = ('' + v).replace(/[_\-+]/g, '.');
            v = v.replace(/([^.\d]+)/g, '.$1.').replace(/\.{2,}/g, '.');
            return (!v.length ? [-8] : v.split('.'));
        },

        numVersion = function(v)
        {
            return !v ? 0 : (isNaN(v) ? vm[v] || -7 : parseInt(v, 10));
        };

    v1 = prepVersion(v1);
    v2 = prepVersion(v2);

    x = Math.max(v1.length, v2.length);
    for (i = 0; i < x; i++)
    {
        if (v1[i] == v2[i])
        {
            continue;
        }

        v1[i] = numVersion(v1[i]);
        v2[i] = numVersion(v2[i]);

        if (v1[i] < v2[i])
        {
            compare = -1;
            break;
        }
        else if (v1[i] > v2[i])
        {
            compare = 1;
            break;
        }
    }

    if (!operator)
    {
        return compare;
    }

    switch (operator)
    {
        case '>':
        case 'gt':
            return (compare > 0);

        case '>=':
        case 'ge':
            return (compare >= 0);

        case '<=':
        case 'le':
            return (compare <= 0);

        case '==':
        case '=':
        case 'eq':
            return (compare === 0);

        case '<>':
        case '!=':
        case 'ne':
            return (compare !== 0);

        case '':
        case '<':
        case 'lt':
            return (compare < 0);

        default:
            return null;
    }
}


/**
 * Comprova si estem connectats a internet
 */
function checkDeviceConnection()
{
    if (navigator.connection)
    {
        return true;
        /*switch(navigator.connection.type)
        {
            case Connection.NONE:
                return false;
                break;

            case Connection.ETHERNET:
            case Connection.WIFI:
            case Connection.CELL_2G:
            case Connection.CELL_3G:
            case Connection.CELL_4G:
            case Connection.UNKNOWN:
            default:
                return true;
                break;
        }*/
    }
    else if (navigator.hasOwnProperty('onLine'))
    {
        return navigator.onLine;
    }

    return true;
}


/**
 * Log a message to console depending on debugging options
 * @param {*} message
 * @param {string} [type=log]
 * @param {bool} [trace=false]
 */
function _log (message, type, trace)
{
    if (DEBUG && typeof console !== 'undefined')
    {
        if (typeof message === 'string')
        {
            var types = ['log', 'debug', 'info', 'warn', 'error'];

            type = type && types.indexOf(type) !== -1 ? type : 'log';

            console[type]('[' + APP.PACKAGE + '] ' + message);
        }
        else
        {
            console.dir(message);
        }

        if (trace)
        {
            console.trace();
        }
    }
}


/**
 * Obtenir un JSON mitjançant ajax
 *
 * @param {String} url - url de l'arxiu ajax (obligatori)
 * @param {String|Object} data - parametres Post per passar a la url (opcional)
 * @param {bool} async - flag és asyncron
 * @param {Function} onSuccess - callback complete
 * @param {Function} onError - callback error
 * @return {jqXHR}
 */
function getJSON(url, data, async, onSuccess, onError)
{
    return $.ajax({
        url: url,
        type: "POST",
        async: async == true,
        data: data,
        dataType: "json",
        success: function(json){
            if (typeof(onSuccess) === 'function'){
                onSuccess(json);
            }
        },
        error: function(jqXHR, textStatus, errorThrown){
            if (typeof(onError) === 'function'){
                onError(textStatus, errorThrown);
            }
        }
    });
}


